﻿/*
author:Ray
date:2017-9-20
*/
var Client = function(domainItem) {
    var _ = this;

    _.ws = null;
    _.guid = '';
    _.clientid = null;
    _.uid = null;
    _.disposed = false;
    _.domainItem = domainItem;

    _.Events = {
        Message: function(command, params) {},
    };

    _.isConnected = function() {
        return _.ws != null && _.ws.readyState == 1;
    };

    //_.IsSuccessConnectedLocalhost = false;

    _.connect = function(done) {
        _.setClientId(_.uid);

        // 当本地服务器能连接上的情况下，则永久不在采用新的
        var wsHost = domainItem.ws;

        if (wsHost == null || wsHost == '') {
            clearTimeout(_.h_connect);
            _.h_connect = setTimeout(function() {
                if (_.disposed) return;

                // 不断尝试重新链接
                console.log('准备重连 ... (' + _.clientid + ')');
                _.connect(done);
            }, 1000);

            return;
        }

        try {
            let pid = localStorage.getItem('ExtmsgPid');
            if (!pid) {
                function generateGUID() {
                    function s4() {
                        return Math.floor((1 + Math.random()) * 0x10000)
                            .toString(16)
                            .substring(1);
                    }
                    return (
                        s4() +
                        s4() +
                        '-' +
                        s4() +
                        '-' +
                        s4() +
                        '-' +
                        s4() +
                        '-' +
                        s4() +
                        s4() +
                        s4()
                    );
                }
                pid = generateGUID();
                localStorage.setItem('ExtmsgPid', pid);
            }

            var url = `${wsHost}/chrome/extmsg?pid=${pid}&host=${wsHost
                .replace('wss://', '')
                .replace('ws://', '')}`;

            console.log('正在连接 ' + url + '... (' + _.clientid + ')');

            // 在创建之前，如果有连接，则先将其释放掉
            if (_.ws) {
                try {
                    _.ws.close();
                } catch {}
            }

            _.ws = new WebSocket(url);
        } catch (e) {
            console.log(
                '连接失败 ' + wsHost + '... (' + _.clientid + ') ' + e.message
            );

            clearTimeout(_.h_connect);
            _.h_connect = setTimeout(function() {
                if (_.disposed) return;

                // 不断尝试重新链接
                _.connect(done);
            }, 10000);
            return;
        }

        //_.ws.onclose = function () {
        //    _.h_connect = setTimeout(function () {
        //        if (_.disposed)
        //            return;

        //        // 不断尝试重新链接
        //        console.log("断开重连 ... (" + _.clientid + ")");
        //        _.connect(done);
        //    }, 1000);
        //}

        // 连接成功
        _.ws.onopen = function() {
            console.log('连接成功 ' + wsHost + '... (' + _.clientid + ')');
            //_.request("login", { clientid: _.clientid });
        };

        _.ws.onmessage = function(e) {
            if (e.data == 1) _.ws.send('2');

            var msg = JSON.parse(e.data);

            if (msg.refuse) {
                console.log(_.clientid + ' refuse');

                _.disposed = true;
                return;
            }

            const ClientDatas = localStorage.getItem('ClientDatas');
            const datas = (ClientDatas && JSON.parse(ClientDatas)) || {};
            const md5 = datas.md5 || '';
            if (md5 && md5 == msg.md5) {
                msg.GATHER_REGEXS = datas.GATHER_REGEXS;
            } else {
                localStorage.setItem('ClientDatas', JSON.stringify(msg));
            }
            _.Events.Message(msg.command, msg);
        };

        if (done) done();
    };

    _.close = function() {
        //console.log("close ... (" + _.clientid + ")");

        if (_.ws) {
            // 关不掉 close 没效果，等待寻找方案
            //_.request("logout", { clientid: _.clientid });

            _.ws.close();
            _.uid = 0;
        }

        _.disposed = true;
    };

    _.init = function(uid) {
        _.uid = uid;
        _.disposed = false;

        //// 设置客户端ID
        //_.setClientId(uid);

        // 开始心跳
        _.connect(function() {
            // 状态验证
            _.connectStatusVerify();
        });
    };

    // 连接状态验证
    _.connectStatusVerify = function() {
        var CLOSED = 3;

        if (_.disposed) return;

        try {
            // 查看连接状态
            if (_.ws == null || _.ws.readyState == CLOSED) {
                console.log('断开重连 ' + _.clientid);
                _.clientid = '';
                _.connect(_.connectStatusVerify);
            }
        } catch (e) {
            //console.log("重新连接:" + e.message);
        }

        // 如果已登录的包含本地信息的时候，不做继续重试
        if (wxwUtil.local) {
            //console.log("本地调试不自动重连");
            //return;
        }

        setTimeout(function() {
            if (_.disposed) return;

            _.connectStatusVerify();
        }, 10000);
    };

    _.setClientId = function(uid) {
        let pid = localStorage.getItem('ExtmsgPid');
        _.guid = pid
            ? pid.split('-')[0]
            : Math.round(Math.random() * 1000000000);

        var newId = uid + '_' + _.guid;

        if (newId != _.clientid) {
            console.log(
                'setClientId ' + newId + ' domainItem.ws:' + domainItem.ws
            );

            _.clientid = uid + '_' + _.guid;
            // 插件如果重启，相关页面要刷新才可以更新cookie
            chrome.cookies.set({
                url: domainItem.ui,
                name: 'wxw_ext_client_id',
                value: _.clientid,
            });

            // 重新登录
            _.request('login', { clientid: _.clientid });
        }
    };

    _.request = function(command, params, done, error) {
        //console.log(command + " " + domainItem.ws );

        wxwUtil.watingFor(
            function() {
                params.version = wxwUtil.VERSION;

                var data = {
                    Command: command,
                    clientid: _.clientid,
                    params: JSON.stringify(params),
                };

                if (_.ws && _.ws.readyState == 1) {
                    const ClientDatas = localStorage.getItem('ClientDatas');
                    const datas =
                        (ClientDatas && JSON.parse(ClientDatas)) || {};
                    if (datas.md5) data.md5 = datas.md5;
                    _.ws.send(JSON.stringify(data));
                }

                if (done) done();
            },
            function() {
                return _.ws && _.ws.readyState == 1;
            },
            { sleep: 500, timeout: 10000 }
        );
    };
};
